@@ -33,7 +33,6 @@
 #include <gtk/gtkimage.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkmenuitem.h>
-#include <gtk/gtknotebook.h>
 #include <gtk/gtkoptionmenu.h>
 #include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtksizegroup.h>
@@ -66,11 +65,36 @@ option_menu_changed (GtkWidget *option_menu,
 	int index;
 
 	index = gtk_option_menu_get_history (GTK_OPTION_MENU (option_menu));
-	
-	gtk_notebook_set_current_page (GTK_NOTEBOOK (editor->notebook),
-				       index);
 
 	editor->active_type = index;
+		
+	gtk_window_set_resizable (GTK_WINDOW (editor), FALSE);	
+	gtk_widget_hide (editor->int_box);
+	gtk_widget_hide (editor->bool_box);
+	gtk_widget_hide (editor->string_box);
+	gtk_widget_hide (editor->float_box);
+	gtk_widget_hide (editor->list_box);
+				       
+	switch (index) {
+		case EDIT_INTEGER:
+			gtk_widget_show_all (editor->int_box);
+			break;
+		case EDIT_BOOLEAN:
+			gtk_widget_show_all (editor->bool_box);
+			break;
+		case EDIT_STRING:
+			gtk_widget_show_all (editor->string_box);
+			break;
+  		case EDIT_FLOAT:
+			gtk_widget_show_all (editor->float_box);
+			break;
+		case EDIT_LIST:
+			gtk_window_set_resizable (GTK_WINDOW (editor), TRUE);
+			gtk_widget_show_all (editor->list_box);
+			break;
+		default:
+			g_assert_not_reached ();
+	}
 }
 
 static void
@@ -178,6 +202,7 @@ update_list_buttons (GConfKeyEditor *editor)
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (editor->list_widget));
 
 	gtk_widget_set_sensitive (editor->remove_button, FALSE);
+	gtk_widget_set_sensitive (editor->edit_button, FALSE);
 	gtk_widget_set_sensitive (editor->go_up_button, FALSE);
 	gtk_widget_set_sensitive (editor->go_down_button, FALSE);
 
@@ -186,6 +211,7 @@ update_list_buttons (GConfKeyEditor *editor)
 
 		selected = gtk_tree_path_get_indices (path)[0];
 
+		gtk_widget_set_sensitive (editor->edit_button, TRUE);
 		gtk_widget_set_sensitive (editor->remove_button, TRUE);
 		gtk_widget_set_sensitive (editor->go_up_button, selected > 0);
 		gtk_widget_set_sensitive (editor->go_down_button, selected < gtk_tree_model_iter_n_children (model, NULL) - 1);
@@ -206,34 +232,31 @@ list_add_clicked (GtkButton *button,
 		  GConfKeyEditor *editor)
 {
 	GtkWidget *dialog;
-	GtkWidget *hbox1, *hbox2;
-	GtkWidget *stock;
+	GtkWidget *hbox;
 	GtkWidget *value_widget;
 	GtkWidget *label;
 	gint response;
 	GtkTreeIter iter;
 	GtkTreeSelection *selection;
 
-	dialog = gtk_dialog_new_with_buttons (_("Add new list entry"),
+	dialog = gtk_dialog_new_with_buttons (_("Add New List Entry"),
                                               GTK_WINDOW (editor),
 					      GTK_DIALOG_MODAL| GTK_DIALOG_DESTROY_WITH_PARENT,
 					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					      GTK_STOCK_OK, GTK_RESPONSE_OK,
 					      NULL);
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (editor)->vbox), 2);
 	
-	hbox1 = gtk_hbox_new (FALSE, 8);
-	gtk_container_set_border_width (GTK_CONTAINER (hbox1), 8);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox1, FALSE, FALSE, 0);
+	hbox = gtk_hbox_new (FALSE, 12);
+	gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, FALSE, FALSE, 0);
 
-	stock = gtk_image_new_from_stock (GTK_STOCK_DIALOG_QUESTION, GTK_ICON_SIZE_DIALOG);
-	gtk_box_pack_start (GTK_BOX (hbox1), stock, FALSE, FALSE, 0);
-
-	hbox2 = gtk_hbox_new (FALSE, 3);
-	gtk_box_pack_start (GTK_BOX (hbox1), hbox2, TRUE, TRUE, 0);
-
-	label = gtk_label_new_with_mnemonic (_("_New list element:"));
-	gtk_box_pack_start (GTK_BOX (hbox2), label, FALSE, FALSE, 0);
+	label = gtk_label_new_with_mnemonic (_("_New list value:"));
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
 	switch (gtk_option_menu_get_history (GTK_OPTION_MENU (editor->list_type_menu))) {
 		case EDIT_INTEGER:
@@ -256,9 +279,9 @@ list_add_clicked (GtkButton *button,
 	}
 
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), value_widget);
-	gtk_box_pack_start (GTK_BOX (hbox2), value_widget, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), value_widget, TRUE, TRUE, 0);
 
-	gtk_widget_show_all (hbox1);
+	gtk_widget_show_all (hbox);
 
 	response = gtk_dialog_run (GTK_DIALOG (dialog));
 
@@ -305,6 +328,108 @@ list_add_clicked (GtkButton *button,
 	gtk_widget_destroy (dialog);
 }
 
+static void
+list_edit_element (GConfKeyEditor *editor)
+{
+	GtkWidget *dialog;
+	GtkWidget *hbox;
+	GtkWidget *value_widget;
+	GtkWidget *label;
+	gint response;
+	GtkTreeIter iter;
+	GtkTreeSelection *selection;
+	GtkTreeModel *model;
+	GConfValue *value = NULL;
+
+	dialog = gtk_dialog_new_with_buttons (_("Edit List Entry"),
+                                              GTK_WINDOW (editor),
+					      GTK_DIALOG_MODAL| GTK_DIALOG_DESTROY_WITH_PARENT,
+					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					      GTK_STOCK_OK, GTK_RESPONSE_OK,
+					      NULL);
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
+	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (editor)->vbox), 2);
+	
+	hbox = gtk_hbox_new (FALSE, 12);
+	gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox, FALSE, FALSE, 0);
+
+	label = gtk_label_new_with_mnemonic (_("_Edit list value:"));
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (editor->list_widget));
+	gtk_tree_selection_get_selected (selection, &model, &iter);
+	gtk_tree_model_get (GTK_TREE_MODEL (model), &iter, 0, &value, -1);
+	switch (gtk_option_menu_get_history (GTK_OPTION_MENU (editor->list_type_menu))) {
+		case EDIT_INTEGER:
+			value_widget = gtk_spin_button_new_with_range (G_MININT, G_MAXINT, 1);
+			gtk_spin_button_set_value (GTK_SPIN_BUTTON (value_widget), gconf_value_get_int (value));
+			break;
+		case EDIT_BOOLEAN:
+			value_widget = gtk_toggle_button_new_with_mnemonic (_("_False"));
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (value_widget), gconf_value_get_bool (value));
+			break;
+		case EDIT_STRING:
+			value_widget = gtk_entry_new ();
+			gtk_entry_set_text (GTK_ENTRY (value_widget), gconf_value_get_string (value));
+			break;
+		default:
+			value_widget = NULL;
+			g_assert_not_reached ();
+	}
+	gconf_value_free (value);
+
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), value_widget);
+	gtk_box_pack_start (GTK_BOX (hbox), value_widget, TRUE, TRUE, 0);
+
+	gtk_widget_show_all (hbox);
+
+	response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+	if (response == GTK_RESPONSE_OK) {
+		GConfValue *value;
+		
+	        value = NULL;
+
+		switch (gtk_option_menu_get_history (GTK_OPTION_MENU (editor->list_type_menu))) {
+			case EDIT_INTEGER:
+				value = gconf_value_new (GCONF_VALUE_INT);
+				gconf_value_set_int (value,
+						     gtk_spin_button_get_value (GTK_SPIN_BUTTON (value_widget)));
+				break;
+
+			case EDIT_BOOLEAN:
+				value = gconf_value_new (GCONF_VALUE_BOOL);
+				gconf_value_set_bool (value,
+						      gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (value_widget)));
+				break;
+
+			case EDIT_STRING:
+		                {
+		                        char *text;
+				
+		                        text = gtk_editable_get_chars (GTK_EDITABLE (value_widget), 0, -1);
+		                        value = gconf_value_new (GCONF_VALUE_STRING);
+		                        gconf_value_set_string (value, text);
+		                        g_free (text);
+		                }
+	                	break;
+			default:
+				g_assert_not_reached ();
+				
+		}
+                gtk_list_store_set (editor->list_model, &iter, 0, value, -1);
+
+	}
+	
+	gtk_widget_destroy (dialog);
+}
+
+
+
 static void
 list_remove_clicked (GtkButton *button,
 		     GConfKeyEditor *editor)
@@ -389,6 +514,22 @@ list_go_down_clicked (GtkButton *button,
 	}
 }
 
+static void
+list_edit_clicked (GtkWidget      *button,
+                   GConfKeyEditor *editor)
+{
+        list_edit_element (editor);
+}
+
+static void
+list_view_row_activated (GtkTreeView       *tree_view,
+                         GtkTreePath       *path,
+                         GtkTreeViewColumn *column,
+                         GConfKeyEditor    *editor)
+{
+        list_edit_element (editor);
+}
+
 static void
 fix_button_align (GtkWidget *button)
 {
@@ -409,7 +550,7 @@ gconf_key_editor_class_init (GConfKeyEditorClass *klass)
 static void
 gconf_key_editor_init (GConfKeyEditor *editor)
 {
-	GtkWidget *hbox, *frame, *framebox;
+	GtkWidget *hbox, *vbox, *framebox;
 	GtkWidget *label, *image;
 	GtkWidget *value_box;
 	GtkWidget *button_box, *button;
@@ -424,128 +565,154 @@ gconf_key_editor_init (GConfKeyEditor *editor)
 				NULL);
 	gtk_dialog_set_default_response (GTK_DIALOG (editor), GTK_RESPONSE_OK);
 	
+	gtk_container_set_border_width (GTK_CONTAINER (editor), 5);
+	gtk_dialog_set_has_separator (GTK_DIALOG (editor), FALSE);
+	gtk_window_set_resizable (GTK_WINDOW (editor), FALSE);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (editor)->vbox), 2);
+
+	vbox = gtk_vbox_new (FALSE, 6);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (editor)->vbox), vbox, TRUE, TRUE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
+	gtk_widget_show_all (vbox);
+
 	size_group = gtk_size_group_new (GTK_SIZE_GROUP_BOTH);
 	
-	editor->path_box = gtk_hbox_new (FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (editor->path_box), 8);
-	label = gtk_label_new (_("Key path:"));
+	editor->path_box = gtk_hbox_new (FALSE, 12);
+	label = gtk_label_new (_("Path:"));
 	gtk_size_group_add_widget (size_group, label);
-	gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
-	gtk_box_pack_start (GTK_BOX (editor->path_box), label, FALSE, FALSE, 4);
+	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	gtk_box_pack_start (GTK_BOX (editor->path_box), label, FALSE, FALSE, 0);
 	editor->path_label = gtk_label_new (NULL);
 	gtk_misc_set_alignment (GTK_MISC (editor->path_label), 0.0, 0.5);
-	gtk_box_pack_start (GTK_BOX (editor->path_box), editor->path_label, TRUE, TRUE, 4);
+	gtk_box_pack_start (GTK_BOX (editor->path_box), editor->path_label, TRUE, TRUE, 0);
 	gtk_widget_show_all (editor->path_box);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (editor)->vbox), editor->path_box, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), editor->path_box, FALSE, FALSE, 0);
 
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (hbox), 8);
-	label = gtk_label_new_with_mnemonic (_("Key _name:"));
+	hbox = gtk_hbox_new (FALSE, 12);
+	label = gtk_label_new_with_mnemonic (_("_Name:"));
 	gtk_size_group_add_widget (size_group, label);
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 4);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 	editor->name_entry = gtk_entry_new ();
+	gtk_widget_set_usize (editor->name_entry, 250, -1);
 	gtk_entry_set_activates_default (GTK_ENTRY (editor->name_entry), TRUE);
-	gtk_box_pack_start (GTK_BOX (hbox), editor->name_entry, TRUE, TRUE, 4);
+	gtk_box_pack_start (GTK_BOX (hbox), editor->name_entry, TRUE, TRUE, 0);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), editor->name_entry);
 	gtk_widget_show_all (hbox);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (editor)->vbox), hbox, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
-	frame = gtk_frame_new (NULL);
-
-	hbox = gtk_hbox_new (FALSE, 2);
+	hbox = gtk_hbox_new (FALSE, 12);
 	label = gtk_label_new_with_mnemonic (_("_Type:"));
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
+	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	gtk_size_group_add_widget (size_group, label);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 	editor->option_menu = gconf_key_editor_create_option_menu (editor);
 	gtk_box_pack_start (GTK_BOX (hbox),
 			    editor->option_menu,
-			    FALSE, FALSE, 2);
+			    TRUE, TRUE, 0);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), editor->option_menu);
 	gtk_widget_show_all (hbox);
-	gtk_frame_set_label_widget (GTK_FRAME (frame), hbox);
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
 	framebox = gtk_vbox_new (FALSE, 6);
 	gtk_widget_show (framebox);
-	gtk_container_add (GTK_CONTAINER (frame), framebox);
+	gtk_box_pack_start (GTK_BOX (vbox), framebox, FALSE, FALSE, 0);
 
-	editor->non_writable_label = gtk_hbox_new (FALSE, 3);
+	editor->non_writable_label = gtk_hbox_new (FALSE, 12);
 	gtk_box_pack_start (GTK_BOX (framebox), editor->non_writable_label, FALSE, FALSE, 0);
 
 	image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING, GTK_ICON_SIZE_BUTTON);
 	gtk_widget_show (image);
-	gtk_box_pack_start (GTK_BOX (editor->non_writable_label), image, FALSE, FALSE, 3);
+	gtk_box_pack_start (GTK_BOX (editor->non_writable_label), image, FALSE, FALSE, 0);
 
 	label = gtk_label_new (_("This key is not writable"));
 	gtk_widget_show (label);
-	gtk_box_pack_start (GTK_BOX (editor->non_writable_label), label, FALSE, FALSE, 3);
-
-	editor->notebook = gtk_notebook_new ();
-	gtk_notebook_set_show_tabs (GTK_NOTEBOOK (editor->notebook),
-				    FALSE);
-	gtk_notebook_set_show_border (GTK_NOTEBOOK (editor->notebook),
-				      FALSE);
-	gtk_container_set_border_width (GTK_CONTAINER (editor->notebook), 3);
-	gtk_box_pack_start (GTK_BOX (framebox), editor->notebook, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (editor->non_writable_label), label, FALSE, FALSE, 0);
 
 	editor->active_type = EDIT_INTEGER;
 
-	/* These have to be ordered such that the EDIT_ enum matches
-	 * the page indices
-	 */
-
 	/* EDIT_INTEGER */
-	value_box = gtk_hbox_new (FALSE, 0);
-	label = gtk_label_new_with_mnemonic (_("Key _value:"));
+	editor->int_box = gtk_hbox_new (FALSE, 0);
+	hbox = gtk_hbox_new (FALSE, 12);
+	label = gtk_label_new ("    ");
+	gtk_box_pack_start (GTK_BOX (editor->int_box), label, FALSE, FALSE, 0);
+
+	label = gtk_label_new_with_mnemonic (_("_Value:"));
 	editor->int_widget = gtk_spin_button_new_with_range (G_MININT, G_MAXINT, 1);
 
 	/* Set a nicer default value */
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (editor->int_widget), 0);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), editor->int_widget);
-	gtk_box_pack_start (GTK_BOX (value_box), label, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (value_box), editor->int_widget, TRUE, TRUE, 0);
-	gtk_notebook_append_page (GTK_NOTEBOOK (editor->notebook),
-				  value_box, NULL);
-
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), editor->int_widget, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (editor->int_box), hbox, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), editor->int_box, FALSE, FALSE, 0);
+	gtk_widget_show_all (editor->int_box);
+	
 	/* EDIT_BOOLEAN */
-	value_box = gtk_hbox_new (FALSE, 0);
-	label = gtk_label_new_with_mnemonic (_("Key _value:"));
+	editor->bool_box = gtk_hbox_new (FALSE, 0);
+	hbox = gtk_hbox_new (FALSE, 12);
+	label = gtk_label_new ("    ");
+	gtk_widget_show (label);
+	gtk_box_pack_start (GTK_BOX (editor->bool_box), label, FALSE, FALSE, 0);
+	label = gtk_label_new_with_mnemonic (_("_Value:"));
 	editor->bool_widget = gtk_toggle_button_new_with_mnemonic (_("_False"));
 	g_signal_connect (editor->bool_widget, "toggled",
 			  G_CALLBACK (bool_button_toggled),
 			  editor);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), editor->bool_widget);
-	gtk_box_pack_start (GTK_BOX (value_box), label, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (value_box), editor->bool_widget, TRUE, TRUE, 0);
-	gtk_notebook_append_page (GTK_NOTEBOOK (editor->notebook),
-				  value_box, NULL);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), editor->bool_widget, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (editor->bool_box), hbox, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), editor->bool_box, FALSE, FALSE, 0);
+	gtk_widget_show_all (hbox);
+	gtk_widget_hide (editor->bool_box);
 
 	/* EDIT_STRING */
-	value_box = gtk_hbox_new (FALSE, 0);
-	label = gtk_label_new_with_mnemonic (_("Key _value:"));
+	editor->string_box = gtk_hbox_new (FALSE, 0);
+	hbox = gtk_hbox_new (FALSE, 12);
+	label = gtk_label_new ("    ");
+	gtk_widget_show (label);
+	gtk_box_pack_start (GTK_BOX (editor->string_box), label, FALSE, FALSE, 0);
+	label = gtk_label_new_with_mnemonic (_("_Value:"));
 	editor->string_widget = gtk_entry_new ();
 	gtk_entry_set_activates_default (GTK_ENTRY (editor->string_widget), TRUE);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), editor->string_widget);	
-	gtk_box_pack_start (GTK_BOX (value_box), label, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (value_box), editor->string_widget, TRUE, TRUE, 0);
-	gtk_notebook_append_page (GTK_NOTEBOOK (editor->notebook),
-				  value_box, NULL);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), editor->string_widget, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (editor->string_box), hbox, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), editor->string_box, FALSE, FALSE, 0);
+	gtk_widget_show_all (hbox);
+	gtk_widget_hide (editor->string_box);
 
-	/* EDIT_INTEGER */
-	value_box = gtk_hbox_new (FALSE, 0);
-	label = gtk_label_new_with_mnemonic (_("Key _value:"));
+	/* EDIT_FLOAT */
+	editor->float_box = gtk_hbox_new (FALSE, 0);
+	hbox = gtk_hbox_new (FALSE, 12);
+	label = gtk_label_new ("    ");
+	gtk_widget_show (label);
+	gtk_box_pack_start (GTK_BOX (editor->float_box), label, FALSE, FALSE, 0);
+	label = gtk_label_new_with_mnemonic (_("_Value:"));
 	editor->float_widget = gtk_spin_button_new_with_range (G_MINFLOAT, G_MAXFLOAT, 0.1);
 
 	/* Set a nicer default value */
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (editor->float_widget), 0.0);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), editor->float_widget);
-	gtk_box_pack_start (GTK_BOX (value_box), label, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (value_box), editor->float_widget, TRUE, TRUE, 0);
-	gtk_notebook_append_page (GTK_NOTEBOOK (editor->notebook),
-				  value_box, NULL);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), editor->float_widget, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (editor->float_box), hbox, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), editor->float_box, FALSE, FALSE, 0);
+	gtk_widget_show_all (hbox);
+	gtk_widget_hide (editor->float_box);
 
         /* EDIT_LIST */
-        value_box = gtk_vbox_new (FALSE, 3);
+	editor->list_box = gtk_hbox_new (FALSE, 0);
+	hbox = gtk_hbox_new (FALSE, 12);
+	label = gtk_label_new ("    ");
+	gtk_widget_show (label);
+	gtk_box_pack_start (GTK_BOX (editor->list_box), label, FALSE, FALSE, 0);
 
-	hbox = gtk_hbox_new (FALSE, 3);
+        value_box = gtk_vbox_new (FALSE, 6);
+
+	hbox = gtk_hbox_new (FALSE, 12);
 	label = gtk_label_new_with_mnemonic (_("List _type:"));
 	editor->list_type_menu = gconf_key_editor_create_list_type_menu (editor);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), editor->list_type_menu);
@@ -554,10 +721,10 @@ gconf_key_editor_init (GConfKeyEditor *editor)
         gtk_box_pack_start (GTK_BOX (hbox),
                             editor->list_type_menu, TRUE, TRUE, 0);
 
-	label = gtk_label_new_with_mnemonic (_("Key _value:"));
+	label = gtk_label_new_with_mnemonic (_("_Values:"));
 	gtk_misc_set_alignment(GTK_MISC (label), 0.0, 0.5);
   
-	hbox = gtk_hbox_new (FALSE, 3);
+	hbox = gtk_hbox_new (FALSE, 12);
 
 	sw = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
@@ -569,6 +736,8 @@ gconf_key_editor_init (GConfKeyEditor *editor)
 	editor->list_widget = gtk_tree_view_new_with_model (GTK_TREE_MODEL (editor->list_model));
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (editor->list_widget), FALSE);
 	list_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (editor->list_widget));
+	g_signal_connect (G_OBJECT (editor->list_widget), "row_activated",
+			  G_CALLBACK (list_view_row_activated), editor);
 	g_signal_connect (G_OBJECT (list_selection), "changed",
 			  G_CALLBACK (list_selection_changed), editor);
  	
@@ -580,17 +749,21 @@ gconf_key_editor_init (GConfKeyEditor *editor)
 
 	button_box = gtk_vbutton_box_new ();
 	gtk_button_box_set_layout (GTK_BUTTON_BOX (button_box), GTK_BUTTONBOX_START);
-	gtk_box_set_spacing (GTK_BOX (button_box), 3);
+	gtk_box_set_spacing (GTK_BOX (button_box), 6);
 	button = gtk_button_new_from_stock (GTK_STOCK_ADD);
 	fix_button_align (button);
 	editor->remove_button = gtk_button_new_from_stock (GTK_STOCK_REMOVE);
 	fix_button_align (editor->remove_button);
+	editor->edit_button = gtk_button_new_from_stock (GTK_STOCK_EDIT);
+	fix_button_align (editor->edit_button);
 	editor->go_up_button = gtk_button_new_from_stock (GTK_STOCK_GO_UP);
 	fix_button_align (editor->go_up_button);
 	editor->go_down_button = gtk_button_new_from_stock (GTK_STOCK_GO_DOWN);
 	fix_button_align (editor->go_down_button);
 	g_signal_connect (G_OBJECT (button), "clicked",
 			  G_CALLBACK (list_add_clicked), editor);
+	g_signal_connect (G_OBJECT (editor->edit_button), "clicked",
+			  G_CALLBACK (list_edit_clicked), editor);
 	g_signal_connect (G_OBJECT (editor->remove_button), "clicked",
 			  G_CALLBACK (list_remove_clicked), editor);
 	g_signal_connect (G_OBJECT (editor->go_up_button), "clicked",
@@ -599,6 +772,7 @@ gconf_key_editor_init (GConfKeyEditor *editor)
 			  G_CALLBACK (list_go_down_clicked), editor);
 	gtk_box_pack_start (GTK_BOX (button_box), button, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (button_box), editor->remove_button, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (button_box), editor->edit_button, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (button_box), editor->go_up_button, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (button_box), editor->go_down_button, FALSE, FALSE, 0);
 
@@ -610,13 +784,10 @@ gconf_key_editor_init (GConfKeyEditor *editor)
         gtk_box_pack_start (GTK_BOX (hbox), sw, TRUE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (hbox), button_box, FALSE, FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (sw), editor->list_widget);
-  	gtk_notebook_append_page (GTK_NOTEBOOK (editor->notebook),
-  				  value_box, NULL);
-
-	gtk_container_set_border_width (GTK_CONTAINER (frame), 8);
-	gtk_container_set_border_width (GTK_CONTAINER (editor->notebook), 8);
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (editor)->vbox), frame, TRUE, TRUE, 8);
-	gtk_widget_show_all (frame);
+	gtk_box_pack_start (GTK_BOX (editor->list_box), value_box, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), editor->list_box, TRUE, TRUE, 0);
+	gtk_widget_show_all (value_box);
+	gtk_widget_hide (editor->list_box);
 }
 
 GType
@@ -652,11 +823,11 @@ gconf_key_editor_new (GConfKeyEditorAction action)
 
 	switch (action) {
 	case GCONF_KEY_EDITOR_NEW_KEY:
-		gtk_window_set_title (GTK_WINDOW (dialog), _("New key"));
+		gtk_window_set_title (GTK_WINDOW (dialog), _("New Key"));
 		gtk_widget_grab_focus (dialog->name_entry);
 		break;
 	case GCONF_KEY_EDITOR_EDIT_KEY:
-		gtk_window_set_title (GTK_WINDOW (dialog), _("Edit key"));
+		gtk_window_set_title (GTK_WINDOW (dialog), _("Edit Key"));
 
 		gtk_widget_set_sensitive (dialog->name_entry, FALSE);
 		gtk_widget_hide (dialog->path_box);
@@ -831,7 +1002,11 @@ gconf_key_editor_set_writable (GConfKeyEditor *editor, gboolean writable)
 	else
 		gtk_widget_show (editor->non_writable_label);
 
-	gtk_widget_set_sensitive (editor->notebook, writable);
+	gtk_widget_set_sensitive (editor->bool_box, writable);
+	gtk_widget_set_sensitive (editor->int_box, writable);
+	gtk_widget_set_sensitive (editor->float_box, writable);
+	gtk_widget_set_sensitive (editor->list_box, writable);
+	gtk_widget_set_sensitive (editor->string_box, writable);
 }
 
 G_CONST_RETURN char *
